from nipype.interfaces.fsl import (BET, FLIRT, ConvertXFM, IsotropicSmooth, FNIRT, InvWarp,
                                    ApplyWarp, Split, ApplyMask, ExtractROI, EddyCorrect, DTIFit,
                                    ConvertWarp,BinaryMaths,FAST,ImageStats)
#from nipype.interfaces.io import DataSink
from nipype.interfaces.freesurfer import ReconAll, SampleToSurface
from nipype.pipeline.engine import Node
from os.path import abspath
import os

from SWANi.SWANiWorkflow.SWANiLib import (Workflow_mo, create_probtrackx2_pipeline, Dcm2niix_mo,Orient_mo,
                                  SwapDimensions,segmentHA_mo,Label2Vol_mo,thrROI, AIndex, Zscore,
                                  VenosaCheck, BEDPOSTX5_mo, getn, DilateImage_mo)
from multiprocessing import cpu_count

import SWANi_supplement

class SWANi_wf(Workflow_mo):

    def __init__(self, name, base_dir=None):
        super(SWANi_wf,self).__init__(name, base_dir)
        self.gen_SWANi_wf()

    def add_input_folders(self,SWANiGlobalConfig,check_input,isfreesurfer):

        if not check_input['mr_t13d']:
            return

        isfreesurfer= isfreesurfer and SWANiGlobalConfig.getboolean('MAIN', 'freesurfer')


        ref_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_mr_t13d_folder'])
        self.ref_conv.inputs.source_dir=ref_dir


        self.t1.sink_result(abspath("."),self.ref_BET,'out_file','scene')
        self.t1.sink_result(abspath("."),self.ref_reOrient,'out_file','scene')
        if isfreesurfer:
            self.connect(self.t1, "ref_conv.converted_files", self.freesurfer, "reconAll.T1_files")
            self.reconAll.inputs.subjects_dir=abspath(os.getcwd())

            self.freesurfer.sink_result(abspath("."),self.aparaseg2Volmgz,'vol_label_file','scene')
            self.freesurfer.sink_result(abspath("."),self.reconAll,'pial','scene')
            self.freesurfer.sink_result(abspath("."),self.reconAll,'white','scene')

            isHippoAmygLabels=SWANiGlobalConfig.getboolean('MAIN', 'hippoAmygLabels')
            if isHippoAmygLabels:
                self.freesurfer.connect(self.reconAll,"subjects_dir",self.segmentHA,"subjects_dir")
                self.freesurfer.connect(self.reconAll,"subject_id",self.segmentHA,"subject_id")
                self.freesurfer.sink_result(abspath("."),self.segmentHA,'lh_hippoAmygLabels','scene.segmentHA')
                self.freesurfer.sink_result(abspath("."),self.segmentHA,'rh_hippoAmygLabels','scene.segmentHA')
                # self.freesurfer.connect(self.reconAll,"rawavg",self.segmentHA_lh2Vol,"template_file")
                # self.freesurfer.connect(self.segmentHA,"lh_hippoAmygLabels",self.segmentHA_lh2Vol,"reg_header")
                # self.freesurfer.connect(self.segmentHA,"lh_hippoAmygLabels",self.segmentHA_lh2Vol,"seg_file")
                # self.freesurfer.connect(self.reconAll,"subjects_dir",self.segmentHA_lh2Vol,"subjects_dir")
                # self.freesurfer.connect(self.reconAll,"subject_id",self.segmentHA_lh2Vol,"subject_id")
                # self.freesurfer.sink_result(abspath("."),self.segmentHA_lh2Vol,'vol_label_file','scene.segmentHA')
                #
                # self.freesurfer.connect(self.reconAll,"rawavg",self.segmentHA_rh2Vol,"template_file")
                # self.freesurfer.connect(self.segmentHA,"lh_hippoAmygLabels",self.segmentHA_rh2Vol,"reg_header")
                # self.freesurfer.connect(self.segmentHA,"lh_hippoAmygLabels",self.segmentHA_rh2Vol,"seg_file")
                # self.freesurfer.connect(self.reconAll,"subjects_dir",self.segmentHA_rh2Vol,"subjects_dir")
                # self.freesurfer.connect(self.reconAll,"subject_id",self.segmentHA_rh2Vol,"subject_id")
                # self.freesurfer.sink_result(abspath("."),self.segmentHA_rh2Vol,'vol_label_file','scene.segmentHA')

        if check_input['mr_flair3d']:
            flair_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_mr_flair3d_folder'])
            self.flair_conv.inputs.source_dir=flair_dir
            self.connect(self.t1, "ref_BET.out_file", self.flair, "flair2ref_FLIRT.reference")
            self.flair.sink_result(abspath("."),self.flair2ref_FLIRT,'out_file','scene')

        if check_input['mr_mdc']:
            mdc_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_mr_mdc_folder'])
            self.mdc_conv.inputs.source_dir=mdc_dir
            self.connect(self.t1, "ref_BET.out_file", self.mdc, "mdc2ref_FLIRT.reference")
            self.mdc.sink_result(abspath("."),self.mdc2ref_FLIRT,'out_file','scene')

        if check_input['mr_asl']:
            asl_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_mr_asl_folder'])
            self.asl_conv.inputs.source_dir=asl_dir
            self.connect(self.t1, "ref_BET.out_file", self.asl, "asl2ref_FLIRT.reference")
            self.connect(self.t1, "ref_BET.mask_file", self.asl, "asl_mask.mask_file")
            self.connect(self.t1, "ref_BET.out_file", self.asl, "aslsmooth2ref_FLIRT.reference")
            self.connect(self.sym, "ref2sym_FNIRT.fieldcoeff_file", self.asl, "asl2sym_APPLYWARP.field_file")
            self.connect(self.sym, "swap2sym_FNIRT.fieldcoeff_file", self.asl, "asl_swapped_APPLYWARP.field_file")
            self.connect(self.sym, "ref2sym_INVWARP.inverse_warp", self.asl, "asl_AI2ref.field_file")
            self.connect(self.t1, "ref_BET.out_file", self.asl, "asl_AI2ref.ref_file")
            self.asl.sink_result(abspath("."),self.asl_mask,'out_file','scene')
            self.asl.sink_result(abspath("."),self.asl_AI2ref,'out_file','scene')

            if isfreesurfer:
                self.asl.connect(self.asl_mask, "out_file", self.asl_surf_lh, "source_file")
                self.asl.connect(self.asl_mask, "out_file", self.asl_surf_rh, "source_file")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.asl, "asl_surf_lh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.asl, "asl_surf_lh.subject_id")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.asl, "asl_surf_rh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.asl, "asl_surf_rh.subject_id")
                self.asl.connect(self.asl_AI2ref, "out_file", self.aslAI_surf_lh, "source_file")
                self.asl.connect(self.asl_AI2ref, "out_file", self.aslAI_surf_rh, "source_file")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.asl, "aslAI_surf_lh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.asl, "aslAI_surf_lh.subject_id")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.asl, "aslAI_surf_rh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.asl, "aslAI_surf_rh.subject_id")
                self.asl.sink_result(abspath("."),self.aslAI_surf_lh,'out_file','scene')
                self.asl.sink_result(abspath("."),self.aslAI_surf_rh,'out_file','scene')
                self.asl.sink_result(abspath("."),self.asl_surf_lh,'out_file','scene')
                self.asl.sink_result(abspath("."),self.asl_surf_rh,'out_file','scene')

        if check_input['pet_brain'] and check_input['ct_brain']:
            pet_brain_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_pet_brain_folder'])
            pet_ct_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_ct_brain_folder'])
            self.pet_ct_conv.inputs.source_dir=pet_ct_dir
            self.pet_brain_conv.inputs.source_dir=pet_brain_dir
            self.connect(self.t1, "ref_reOrient.out_file", self.pet, "pet_ct2ref_FLIRT.reference")
            self.connect(self.t1, "ref_reOrient.out_file", self.pet, "pet_brain2ref_FLIRT.reference")
            self.connect(self.sym, "ref2sym_FNIRT.fieldcoeff_file", self.pet, "pet_brain_sm_APPLYWARP.field_file")
            self.connect(self.sym, "swap2sym_FNIRT.fieldcoeff_file", self.pet, "pet_brain_swapped_APPLYWARP.field_file")
            self.connect(self.sym, "ref2sym_INVWARP.inverse_warp", self.pet, "pet_AI2ref.field_file")
            self.connect(self.t1, "ref_BET.out_file", self.pet, "pet_AI2ref.ref_file")
            self.connect(self.t1, "ref_BET.mask_file", self.pet, "pet_AI_mask.mask_file")
            self.connect(self.t1, "ref_BET.mask_file", self.pet, "pet_zscore_mask.mask_file")
            self.pet.sink_result(abspath("."),self.pet_brain_SMOOTH,'out_file','scene')
            self.pet.sink_result(abspath("."),self.pet_AI_mask,'out_file','scene')
            self.pet.sink_result(abspath("."),self.pet_zscore_mask,'out_file','scene')
            if isfreesurfer:
                self.pet.connect(self.pet_brain_SMOOTH, "out_file", self.pet_surf_lh, "source_file")
                self.pet.connect(self.pet_brain_SMOOTH, "out_file", self.pet_surf_rh, "source_file")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.pet, "pet_surf_lh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.pet, "pet_surf_lh.subject_id")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.pet, "pet_surf_rh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.pet, "pet_surf_rh.subject_id")
                self.pet.connect(self.pet_AI_mask, "out_file", self.petAI_surf_lh, "source_file")
                self.pet.connect(self.pet_AI_mask, "out_file", self.petAI_surf_rh, "source_file")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.pet, "petAI_surf_lh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.pet, "petAI_surf_lh.subject_id")
                self.connect(self.freesurfer, "reconAll.subjects_dir", self.pet, "petAI_surf_rh.subjects_dir")
                self.connect(self.freesurfer, "reconAll.subject_id", self.pet, "petAI_surf_rh.subject_id")
                self.connect(self.freesurfer, "bgtROI.out_file", self.pet, "pet_zscore.ROI_file")
                self.pet.sink_result(abspath("."),self.petAI_surf_lh,'out_file','scene')
                self.pet.sink_result(abspath("."),self.petAI_surf_rh,'out_file','scene')
                self.pet.sink_result(abspath("."),self.pet_surf_lh,'out_file','scene')
                self.pet.sink_result(abspath("."),self.pet_surf_rh,'out_file','scene')

        if check_input['mr_venosa']:
            venosa_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_mr_venosa_folder'])
            self.venosa_conv.inputs.source_dir=venosa_dir
            self.connect(self.t1, "ref_BET.out_file", self.venosa, "venosa_modulo2ref_FLIRT.reference")
            self.connect(self.t1, "ref_BET.out_file", self.venosa, "venosa2ref_FLIRT.reference")
            self.venosa.sink_result(abspath("."),self.venosa2ref_FLIRT,'out_file','scene')

        if check_input['mr_dti']:
            dti_dir = abspath(SWANiGlobalConfig['DEFAULTFOLDERS']['default_mr_dti_folder'])
            self.dti_conv.inputs.source_dir=dti_dir
            self.connect(self.t1, "ref_BET.out_file", self.dti_preproc, "diff2ref_FLIRT.reference")
            self.connect(self.mni, "ref2mni_FLIRT.out_matrix_file", self.dti_preproc, "diff2mni_convert.in_file")
            self.connect(self.mni, "ref2mni_FNIRT.fieldcoeff_file", self.dti_preproc, "diff2mni_convertwarp.warp1")
            self.connect(self.mni, "ref2mni_INVWARP.inverse_warp", self.dti_preproc, "mni2diff_convertwarp.warp1")

            tractWf_list={}
            for tract in SWANiGlobalConfig['DEFAULTTRACTS'].keys():
                if not SWANiGlobalConfig.getboolean('DEFAULTTRACTS',tract): continue
                tractWf_list[tract]=create_probtrackx2_pipeline('tract_'+tract,tract,abspath("."))
                self.connect(self.dti_preproc, "dti_bedpostx.merged_fsamples", tractWf_list[tract], "inputnode.fsamples")
                self.connect(self.dti_preproc, "nodif_BET.mask_file", tractWf_list[tract], "inputnode.mask")
                self.connect(self.dti_preproc, "dti_bedpostx.merged_phsamples", tractWf_list[tract], "inputnode.phsamples")
                self.connect(self.dti_preproc, "dti_bedpostx.merged_thsamples", tractWf_list[tract], "inputnode.thsamples")
                self.connect(self.dti_preproc, "mni2diff_convertwarp.out_file", tractWf_list[tract], "inputnode.xfm")
                self.connect(self.dti_preproc, "diff2mni_convertwarp.out_file", tractWf_list[tract], "inputnode.inv_xfm")
                self.connect(self.t1, "ref_BET.out_file", tractWf_list[tract], "inputnode.ref")
                self.connect(self.mni, "ref2mni_INVWARP.inverse_warp", tractWf_list[tract], "inputnode.mni2ref_warp")

    def gen_SWANi_wf(self):

        sym_template = SWANi_supplement.sym_template

        mni = abspath(os.path.join(os.environ["FSL_DIR"],'data/standard/MNI152_T1_2mm_brain.nii.gz'))
        mni1 = abspath(os.path.join(os.environ["FSL_DIR"],'data/standard/MNI152_T1_1mm_brain.nii.gz'))

        max_cpu= max(int((cpu_count()-2)/2),1)

        #ELABORAZIONE T1 3D
        #coversione dicom->nifti
        self.ref_conv = Node(Dcm2niix_mo(),name='ref_conv')
        self.ref_conv.inputs.crop=True
        self.ref_conv.inputs.out_filename ="ref"
        #orientamento in convenzione radiologica
        self.ref_reOrient=Node(Orient_mo(),name='ref_reOrient')
        #rimozione dello scalpo
        self.ref_BET = Node(BET(),name='ref_BET')
        self.ref_BET.inputs.frac = 0.5
        self.ref_BET.inputs.mask = True
        self.ref_BET.inputs.robust = True
        self.ref_BET.inputs.threshold = True

        #REGISTRAZIONE AD ALTANTE MNI
        #registrazione lineare
        ref2mni_FLIRT = Node(FLIRT(), name='ref2mni_FLIRT')
        ref2mni_FLIRT.inputs.reference = mni
        ref2mni_FLIRT.inputs.cost = "mutualinfo"
        ref2mni_FLIRT.inputs.searchr_x = [-90,90]
        ref2mni_FLIRT.inputs.searchr_y = [-90,90]
        ref2mni_FLIRT.inputs.searchr_z = [-90,90]
        ref2mni_FLIRT.inputs.dof = 12
        ref2mni_FLIRT.inputs.cost = "corratio"
        ref2mni_FLIRT.inputs.out_matrix_file  = "ref2mni.mat"
        #registrazione non lineare
        ref2mni_FNIRT = Node(FNIRT(), name='ref2mni_FNIRT')
        ref2mni_FNIRT.inputs.ref_file = mni
        ref2mni_FNIRT.inputs.fieldcoeff_file = True
        #matrice inversa da atlante mni a ref
        ref2mni_INVWARP = Node(InvWarp(), name='ref2mni_INVWARP')

        #REGISTRAZIONE AD ALTANTE MNI1mm
        #registrazione lineare
        ref2mni1_FLIRT = Node(FLIRT(), name='ref2mni1_FLIRT')
        ref2mni1_FLIRT.inputs.reference = mni1
        ref2mni1_FLIRT.inputs.cost = "mutualinfo"
        ref2mni1_FLIRT.inputs.searchr_x = [-90,90]
        ref2mni1_FLIRT.inputs.searchr_y = [-90,90]
        ref2mni1_FLIRT.inputs.searchr_z = [-90,90]
        ref2mni1_FLIRT.inputs.dof = 12
        ref2mni1_FLIRT.inputs.cost = "corratio"
        ref2mni1_FLIRT.inputs.out_matrix_file  = "ref2mni1.mat"
        #registrazione non lineare
        ref2mni1_FNIRT = Node(FNIRT(), name='ref2mni1_FNIRT')
        ref2mni1_FNIRT.inputs.ref_file = mni
        ref2mni1_FNIRT.inputs.fieldcoeff_file = True
        #matrice inversa da atlante mni a ref
        ref2mni1_INVWARP = Node(InvWarp(), name='ref2mni1_INVWARP')

        #REGISTRAZIONE AD ALTANTE SIMMETRICO PER EVENTUALI ASIMMETRY index
        #registrazione lineare
        ref2sym_FLIRT = Node(FLIRT(), name='ref2sym_FLIRT')
        ref2sym_FLIRT.inputs.reference = sym_template
        ref2sym_FLIRT.inputs.cost = "mutualinfo"
        ref2sym_FLIRT.inputs.searchr_x = [-90,90]
        ref2sym_FLIRT.inputs.searchr_y = [-90,90]
        ref2sym_FLIRT.inputs.searchr_z = [-90,90]
        ref2sym_FLIRT.inputs.dof = 12
        ref2sym_FLIRT.inputs.cost = "corratio"
        ref2sym_FLIRT.inputs.out_matrix_file  = "ref2sym.mat"
        #registrazione non lineare
        ref2sym_FNIRT = Node(FNIRT(), name='ref2sym_FNIRT')
        ref2sym_FNIRT.inputs.ref_file = sym_template
        ref2sym_FNIRT.inputs.fieldcoeff_file = True
        #matrice inversa da atlante simmetrico a ref
        ref2sym_INVWARP = Node(InvWarp(), name='ref2sym_INVWARP')
        #immagine ribaltata in RL del soggetto nello spazio dell'atlante simmetrico
        sym_SWAP=Node(SwapDimensions(), name='sym_SWAP')
        sym_SWAP.inputs.out_file="sym_ref_brain_swapped.nii.gz"
        sym_SWAP.inputs.new_dims=("-x","y","z")
        #trasformazione lineare dell'immagine ribaltata sull'originale
        swap2sym_FLIRT = Node(FLIRT(), name='swap2sym_FLIRT')
        swap2sym_FLIRT.inputs.cost = "mutualinfo"
        swap2sym_FLIRT.inputs.searchr_x = [-90,90]
        swap2sym_FLIRT.inputs.searchr_y = [-90,90]
        swap2sym_FLIRT.inputs.searchr_z = [-90,90]
        swap2sym_FLIRT.inputs.dof = 6
        swap2sym_FLIRT.inputs.interp = "trilinear"
        swap2sym_FLIRT.inputs.out_matrix_file  = "swap2sym.mat"
        #trasformazione non lineare dell'immagine ribaltata sull'originale
        swap2sym_FNIRT = Node(FNIRT(), name='swap2sym_FNIRT')
        swap2sym_FNIRT.inputs.fieldcoeff_file = True
        swap2sym_FNIRT.inputs.ref_file=sym_template

        #ELABORAZIONE FREESURFER
        #recon all
        self.reconAll=Node(ReconAll(), name='reconAll')
        self.reconAll.inputs.subject_id="FS"
        self.reconAll.inputs.openmp=max_cpu
        self.reconAll.inputs.parallel=True
        self.reconAll.inputs.directive = 'all'
        #sposto aparcaseg nello spazio ref
        self.aparaseg2Volmgz = Node(Label2Vol_mo(),name="aparaseg2Volmgz")
        self.aparaseg2Volmgz.inputs.vol_label_file="./r-aparc_aseg.mgz"
        aparaseg2Volnii = Node(Label2Vol_mo(),name="aparaseg2Volnii")
        aparaseg2Volnii.inputs.vol_label_file="r-aparc_aseg.nii.gz"
        #estrazione ROI sostanza bianca
        lhwmROI = Node(thrROI(),name='lhwmROI')
        lhwmROI.inputs.seg_val_min=2
        lhwmROI.inputs.seg_val_max=2
        lhwmROI.inputs.out_file="lhwmROI.nii.gz"
        rhwmROI = Node(thrROI(),name='rhwmROI')
        rhwmROI.inputs.seg_val_min=41
        rhwmROI.inputs.seg_val_max=41
        rhwmROI.inputs.out_file="rhwmROI.nii.gz"
        wmROI = Node(BinaryMaths(),name='wmROI')
        wmROI.inputs.operation="add"
        wmROI.inputs.out_file="wmROI.nii.gz"
        #estrazione ROI bgt
        lhbgtROI = Node(thrROI(),name='lhbgtROI')
        lhbgtROI.inputs.seg_val_min=10
        lhbgtROI.inputs.seg_val_max=13
        lhbgtROI.inputs.out_file="lhbgtROI.nii.gz"
        rhbgtROI = Node(thrROI(),name='rhbgtROI')
        rhbgtROI.inputs.seg_val_min=49
        rhbgtROI.inputs.seg_val_max=52
        rhbgtROI.inputs.out_file="rhbgtROI.nii.gz"
        bgtROI = Node(BinaryMaths(),name='bgtROI')
        bgtROI.inputs.operation="add"
        bgtROI.inputs.out_file="bgtROI.nii.gz"
        #segmentazione ippocampo e amigdala
        self.segmentHA = Node(segmentHA_mo(),name="segmentHA_mo")
        self.segmentHA.inputs.num_threads=max_cpu
        #sposto la segmentazione ippocampo e amigdala nello spazio ref
        # self.segmentHA_lh2Vol = Node(Label2Vol_mo(),name="segmentHA_lh2Vol")
        # self.segmentHA_lh2Vol.inputs.vol_label_file="./r-lh.hippoAmygLabels-T1.mgz"
        # self.segmentHA_rh2Vol = Node(Label2Vol_mo(),name="segmentHA_rh2Vol")
        # self.segmentHA_rh2Vol.inputs.vol_label_file="./r-rh.hippoAmygLabels-T1.mgz"


        #ELABORAZIONE script_DoMAP
        #segmentazione con fast
        doMAP_FAST = Node(FAST(),name="doMAP_FAST")
        doMAP_FAST.inputs.img_type=1
        doMAP_FAST.inputs.number_classes=3
        doMAP_FAST.inputs.hyper=0.1
        doMAP_FAST.inputs.bias_lowpass=40
        doMAP_FAST.inputs.output_biascorrected=True
        doMAP_FAST.inputs.bias_iters=4
        #flair in atlante MNI1
        doMAP_flair2mni1 = Node(ApplyWarp(), name="doMAP_flair2mni1")
        doMAP_flair2mni1.inputs.ref_file = mni1
        #t1_restore in atlante MNI1
        doMAP_restore2mni1 = Node(ApplyWarp(), name="doMAP_restore2mni1")
        doMAP_restore2mni1.inputs.ref_file = mni1
        #GM in atlante MNI1
        doMAP_gm2mni1 = Node(ApplyWarp(), name="doMAP_gm2mni1")
        doMAP_gm2mni1.inputs.ref_file = mni1
        #WM in atlante MNI1
        doMAP_wm2mni1 = Node(ApplyWarp(), name="doMAP_wm2mni1")
        doMAP_wm2mni1.inputs.ref_file = mni1
        #divido FLAIR/T1
        doMAP_flairDIVref = Node(BinaryMaths(),name="doMAP_flairDIVref")
        doMAP_flairDIVref.inputs.operation="div"
        #rimuovo il cervelletto dalla flair/t1
        doMAP_cortexMask = Node(ApplyMask(), name="doMAP_cortexMask")
        doMAP_cortexMask.inputs.mask_file=SWANi_supplement.cortex_mas
        #creazione maschere gm e wn su t1_restore in MNI1
        doMAP_gmMask = Node(ApplyMask(), name="doMAP_gmMask")
        doMAP_wmMask = Node(ApplyMask(), name="doMAP_wmMask")
        #calcolo media e dev standard per gm e wm
        doMAP_gm_mean = Node(ImageStats(), name="doMAP_gm_mean")
        doMAP_gm_mean.inputs.op_string="-M"
        doMAP_wm_mean = Node(ImageStats(), name="doMAP_wm_mean")
        doMAP_wm_mean.inputs.op_string="-M"
        doMAP_gm_std = Node(ImageStats(), name="doMAP_gm_std")
        doMAP_gm_std.inputs.op_string="-S"
        doMAP_wm_std = Node(ImageStats(), name="doMAP_wm_std")
        doMAP_wm_std.inputs.op_string="-S"
        #maschera generata da soglia per media e dev standard su immagine divisa (perchè mai???)
        doMAP_binaryFLAIR = Node(thrROI(), name='doMAP_binaryFLAIR')
        doMAP_binaryFLAIR.inputs.out_file="binary_flair.nii.gz"
        #convolutional flair_reOrient
        doMAP_convolution_flair = Node(DilateImage_mo(),name="doMAP_convolution_flair")
        doMAP_convolution_flair.inputs.args="-fmean"
        doMAP_convolution_flair.inputs.kernel_shape="boxv"
        doMAP_convolution_flair.inputs.kernel_size=5
        doMAP_convolution_flair.inputs.out_file="convolution_flair.nii.gz"
        #calcolo junction e relativo zscore
        doMAP_junction = Node(BinaryMaths(), name="doMAP_junction")
        doMAP_junction.inputs.operation="sub"
        doMAP_junction.inputs.operand_file=SWANi_supplement.mean_flair
        doMAP_junction.inputs.out_file="junction_flair.nii.gz"
        doMAP_junctionz = Node(BinaryMaths(), name="doMAP_junctionz")
        doMAP_junctionz.inputs.operation="div"
        doMAP_junctionz.inputs.operand_file=SWANi_supplement.std_final_flair
        doMAP_junctionz.inputs.out_file="junctionZ_flair.nii.gz"


        #ELABORAZIONE FLAIR
        #conversione dicom->nifti
        self.flair_conv = Node(Dcm2niix_mo(),name='flair_conv')
        self.flair_conv.inputs.crop=True
        self.flair_conv.inputs.out_filename ="flair"
        #orientamento in convenzione radiologica
        flair_reOrient=Node(Orient_mo(),name='flair_reOrient')
        #rimozione scalpo
        flair_BET = Node(BET(),name='flair_BET')
        flair_BET.inputs.frac = 0.5
        flair_BET.inputs.robust = True
        flair_BET.inputs.threshold = True
        #trasformazione lineare nello spazio ref
        self.flair2ref_FLIRT = Node(FLIRT(), name='flair2ref_FLIRT')
        self.flair2ref_FLIRT.inputs.out_file  = "r-flair_brain.nii.gz"
        self.flair2ref_FLIRT.inputs.out_matrix_file  = "flair2ref.mat"
        self.flair2ref_FLIRT.inputs.cost = "mutualinfo"
        self.flair2ref_FLIRT.inputs.searchr_x = [-90,90]
        self.flair2ref_FLIRT.inputs.searchr_y = [-90,90]
        self.flair2ref_FLIRT.inputs.searchr_z = [-90,90]
        self.flair2ref_FLIRT.inputs.dof = 6
        self.flair2ref_FLIRT.inputs.interp = "trilinear"

        #ELABORAZIONE MDC
        #conversione dicom->nifti
        self.mdc_conv = Node(Dcm2niix_mo(),name='mdc_conv')
        self.mdc_conv.inputs.crop=True
        self.mdc_conv.inputs.out_filename ="mdc"
        #orientamento in convenzione radiologica
        mdc_reOrient=Node(Orient_mo(),name='mdc_reOrient')
        #rimozione scalpo
        mdc_BET = Node(BET(),name='mdc_BET')
        mdc_BET.inputs.frac = 0.3
        mdc_BET.inputs.robust = True
        mdc_BET.inputs.threshold = True
        #trasformazione lineare nello spazio ref
        self.mdc2ref_FLIRT = Node(FLIRT(), name='mdc2ref_FLIRT')
        self.mdc2ref_FLIRT.inputs.out_file  = "r-mdc_brain.nii.gz"
        self.mdc2ref_FLIRT.inputs.out_matrix_file  = "mdc2ref.mat"
        self.mdc2ref_FLIRT.inputs.cost = "mutualinfo"
        self.mdc2ref_FLIRT.inputs.searchr_x = [-90,90]
        self.mdc2ref_FLIRT.inputs.searchr_y = [-90,90]
        self.mdc2ref_FLIRT.inputs.searchr_z = [-90,90]
        self.mdc2ref_FLIRT.inputs.dof = 6
        self.mdc2ref_FLIRT.inputs.interp = "trilinear"

        #ELABORAZIONE ASL
        #conversione dicom->nifti
        self.asl_conv = Node(Dcm2niix_mo(),name='asl_conv')
        self.asl_conv.inputs.out_filename ="asl"
        #orientamento in convenzione radiologica
        asl_reOrient=Node(Orient_mo(),name='asl_reOrient')
        #smoothing gaussiano
        asl_SMOOTH = Node(IsotropicSmooth(),name='asl_SMOOTH')
        asl_SMOOTH.inputs.sigma = 4
        #calcolo matrice di trasformazione nello spazio ref
        asl2ref_FLIRT = Node(FLIRT(), name='asl2ref_FLIRT')
        asl2ref_FLIRT.inputs.cost = "mutualinfo"
        asl2ref_FLIRT.inputs.searchr_x = [-90,90]
        asl2ref_FLIRT.inputs.searchr_y = [-90,90]
        asl2ref_FLIRT.inputs.searchr_z = [-90,90]
        asl2ref_FLIRT.inputs.dof = 6
        asl2ref_FLIRT.inputs.interp = "trilinear"
        #trasposizione del volume smooth nello spazio ref
        aslsmooth2ref_FLIRT=Node(FLIRT(), name='aslsmooth2ref_FLIRT')
        aslsmooth2ref_FLIRT.inputs.out_file  = "r-asl.nii.gz"
        aslsmooth2ref_FLIRT.inputs.interp = "trilinear"
        #rimozione scalpo
        self.asl_mask = Node(ApplyMask(),name='asl_mask')
        self.asl_mask.inputs.out_file='r-asl.nii.gz'
        #trasformazione non lineare delle immagini asl nell'atlante simmetrico
        asl2sym_APPLYWARP = Node(ApplyWarp(), name='asl2sym_APPLYWARP')
        asl2sym_APPLYWARP.inputs.ref_file=sym_template
        #immagine ribaltata in RL della asl nello spazio dell'atlante simmetrico
        aslsym_SWAP=Node(SwapDimensions(), name='aslsym_SWAP')
        aslsym_SWAP.inputs.out_file="asl_sym_swapped.nii.gz"
        aslsym_SWAP.inputs.new_dims=("-x","y","z")
        #trasformazione non lineare da immagine ribaltata a immagine simmetrica
        asl_swapped_APPLYWARP = Node(ApplyWarp(), name='asl_swapped_APPLYWARP')
        asl_swapped_APPLYWARP.inputs.ref_file=sym_template
        #calcolo asimmetry index
        asl_AI = Node(AIndex(), name="asl_AI")
        asl_AI.inputs.out_file="aslAI.nii.gz"
        #trasformazione non lineare da atlante simmetrico a ref
        self.asl_AI2ref = Node(ApplyWarp(),name="asl_AI2ref")
        self.asl_AI2ref.inputs.out_file="r-asl_AI.nii.gz"
        #PROIEZIONE DELLA asl SULLA SUPERFICIE PIALE DI FREESURFER
        self.asl_surf_lh = Node(SampleToSurface(), name='asl_surf_lh')
        self.asl_surf_lh.inputs.hemi='lh'
        self.asl_surf_lh.inputs.out_file="asl_surf_lh.mgz"
        self.asl_surf_lh.inputs.cortex_mask=True
        self.asl_surf_lh.inputs.reg_header=True
        self.asl_surf_lh.inputs.sampling_method = "point"
        self.asl_surf_lh.inputs.sampling_range = 0.5
        self.asl_surf_lh.inputs.sampling_units = "frac"
        self.asl_surf_rh = Node(SampleToSurface(), name='asl_surf_rh')
        self.asl_surf_rh.inputs.hemi='rh'
        self.asl_surf_rh.inputs.out_file="asl_surf_rh.mgz"
        self.asl_surf_rh.inputs.cortex_mask=True
        self.asl_surf_rh.inputs.reg_header=True
        self.asl_surf_rh.inputs.sampling_method = "point"
        self.asl_surf_rh.inputs.sampling_range = 0.5
        self.asl_surf_rh.inputs.sampling_units = "frac"
        #PROIEZIONE DELLA asl AI SULLA SUPERFICIE PIALE DI FREESURFER
        self.aslAI_surf_lh = Node(SampleToSurface(), name='aslAI_surf_lh')
        self.aslAI_surf_lh.inputs.hemi='lh'
        self.aslAI_surf_lh.inputs.out_file="aslAI_surf_lh.mgz"
        self.aslAI_surf_lh.inputs.cortex_mask=True
        self.aslAI_surf_lh.inputs.reg_header=True
        self.aslAI_surf_lh.inputs.sampling_method = "point"
        self.aslAI_surf_lh.inputs.sampling_range = 0.5
        self.aslAI_surf_lh.inputs.sampling_units = "frac"
        self.aslAI_surf_rh = Node(SampleToSurface(), name='aslAI_surf_rh')
        self.aslAI_surf_rh.inputs.hemi='rh'
        self.aslAI_surf_rh.inputs.out_file="aslAI_surf_rh.mgz"
        self.aslAI_surf_rh.inputs.cortex_mask=True
        self.aslAI_surf_rh.inputs.reg_header=True
        self.aslAI_surf_rh.inputs.sampling_method = "point"
        self.aslAI_surf_rh.inputs.sampling_range = 0.5
        self.aslAI_surf_rh.inputs.sampling_units = "frac"


        #ELABORAZIONE PET
        #conversione immagine TC dicom->nifti
        self.pet_ct_conv = Node(Dcm2niix_mo(), name='pet_ct_conv')
        self.pet_ct_conv.inputs.out_filename ="pet_ct"
        #conversione immagine PET dicom->nifti
        self.pet_brain_conv = Node(Dcm2niix_mo(), name='pet_brain_conv')
        self.pet_brain_conv.inputs.out_filename ="pet_brain"
        #trasformazione lineare immagine TC su ref
        pet_ct2ref_FLIRT = Node(FLIRT(), name='pet_ct2ref_FLIRT')
        pet_ct2ref_FLIRT.inputs.out_file  = "r-pet_ct.nii.gz"
        pet_ct2ref_FLIRT.inputs.out_matrix_file  = "ct2ref.mat"
        pet_ct2ref_FLIRT.inputs.cost = "mutualinfo"
        pet_ct2ref_FLIRT.inputs.searchr_x = [-90,90]
        pet_ct2ref_FLIRT.inputs.searchr_y = [-90,90]
        pet_ct2ref_FLIRT.inputs.searchr_z = [-90,90]
        pet_ct2ref_FLIRT.inputs.dof = 6
        pet_ct2ref_FLIRT.inputs.interp = "trilinear"
        #trasformazione lineare immagine PET su TC
        pet_brain2ct_FLIRT = Node(FLIRT(), name='pet_brain2ct_FLIRT')
        pet_brain2ct_FLIRT.inputs.out_matrix_file  = "pet2ct.mat"
        pet_brain2ct_FLIRT.inputs.cost = "mutualinfo"
        pet_brain2ct_FLIRT.inputs.searchr_x = [-90,90]
        pet_brain2ct_FLIRT.inputs.searchr_y = [-90,90]
        pet_brain2ct_FLIRT.inputs.searchr_z = [-90,90]
        pet_brain2ct_FLIRT.inputs.dof = 6
        pet_brain2ct_FLIRT.inputs.interp = "trilinear"
        #concatenazione delle due trasformazioni precedenti
        pet_convert_XFM = Node(ConvertXFM(), name='pet_convert_XFM')
        pet_convert_XFM.inputs.concat_xfm = True
        pet_convert_XFM.inputs.out_file = "pet2ref.mat"
        #trasformazione lineare immagine PET su ref
        pet_brain2ref_FLIRT = Node(FLIRT(), name='pet_brain2ref_FLIRT')
        pet_brain2ref_FLIRT.inputs.out_file  = "r-pet_brain.nii.gz"
        pet_brain2ref_FLIRT.inputs.apply_xfm = True
        pet_brain2ref_FLIRT.inputs.interp = "trilinear"
        #smoothing gaussiano
        self.pet_brain_SMOOTH = Node(IsotropicSmooth(),name='pet_brain_SMOOTH')
        self.pet_brain_SMOOTH.inputs.sigma = 4
        #trasformazione non lineare delle immagini PET nell'atlante simmetrico
        pet_brain_sm_APPLYWARP = Node(ApplyWarp(), name='pet_brain_sm_APPLYWARP')
        pet_brain_sm_APPLYWARP.inputs.ref_file=sym_template
        #immagine ribaltata in RL della PET nello spazio dell'atlante simmetrico
        petsym_SWAP=Node(SwapDimensions(), name='petsym_SWAP')
        petsym_SWAP.inputs.out_file="pet_brain_sym_swapped.nii.gz"
        petsym_SWAP.inputs.new_dims=("-x","y","z")
        #trasformazione non lineare da immagine ribaltata a immagine simmetrica
        pet_brain_swapped_APPLYWARP = Node(ApplyWarp(), name='pet_brain_swapped_APPLYWARP')
        pet_brain_swapped_APPLYWARP.inputs.ref_file=sym_template
        #calcolo asimmetry index
        pet_AI = Node(AIndex(), name="pet_AI")
        pet_AI.inputs.out_file="petAI.nii.gz"
        #trasformazione non lineare da atlante simmetrico a ref
        pet_AI2ref = Node(ApplyWarp(),name="pet_AI2ref")
        pet_AI2ref.inputs.out_file="r-petAI.nii.gz"
        #applico la maschera per eliminare il fondo
        self.pet_AI_mask = Node(ApplyMask(),name="pet_AI_mask")
        self.pet_AI_mask.inputs.out_file="r-pet_brain_AI.nii.gz"
        pet_zscore = Node(Zscore(),name="pet_zscore")
        self.pet_zscore_mask = Node(ApplyMask(),name="pet_zscore_mask")
        self.pet_zscore_mask.inputs.out_file="r-pet_brain_z.nii.gz"
        #PROIEZIONE DELLA PET SULLA SUPERFICIE PIALE DI FREESURFER
        self.pet_surf_lh = Node(SampleToSurface(), name='pet_surf_lh')
        self.pet_surf_lh.inputs.hemi='lh'
        self.pet_surf_lh.inputs.out_file="pet_surf_lh.mgz"
        self.pet_surf_lh.inputs.cortex_mask=True
        self.pet_surf_lh.inputs.reg_header=True
        self.pet_surf_lh.inputs.sampling_method = "point"
        self.pet_surf_lh.inputs.sampling_range = 0.5
        self.pet_surf_lh.inputs.sampling_units = "frac"
        self.pet_surf_rh = Node(SampleToSurface(), name='pet_surf_rh')
        self.pet_surf_rh.inputs.hemi='rh'
        self.pet_surf_rh.inputs.out_file="pet_surf_rh.mgz"
        self.pet_surf_rh.inputs.cortex_mask=True
        self.pet_surf_rh.inputs.reg_header=True
        self.pet_surf_rh.inputs.sampling_method = "point"
        self.pet_surf_rh.inputs.sampling_range = 0.5
        self.pet_surf_rh.inputs.sampling_units = "frac"
        #PROIEZIONE DELLA PET AI SULLA SUPERFICIE PIALE DI FREESURFER
        self.petAI_surf_lh = Node(SampleToSurface(), name='petAI_surf_lh')
        self.petAI_surf_lh.inputs.hemi='lh'
        self.petAI_surf_lh.inputs.out_file="petAI_surf_lh.mgz"
        self.petAI_surf_lh.inputs.cortex_mask=True
        self.petAI_surf_lh.inputs.reg_header=True
        self.petAI_surf_lh.inputs.sampling_method = "point"
        self.petAI_surf_lh.inputs.sampling_range = 0.5
        self.petAI_surf_lh.inputs.sampling_units = "frac"
        self.petAI_surf_rh = Node(SampleToSurface(), name='petAI_surf_rh')
        self.petAI_surf_rh.inputs.hemi='rh'
        self.petAI_surf_rh.inputs.out_file="petAI_surf_rh.mgz"
        self.petAI_surf_rh.inputs.cortex_mask=True
        self.petAI_surf_rh.inputs.reg_header=True
        self.petAI_surf_rh.inputs.sampling_method = "point"
        self.petAI_surf_rh.inputs.sampling_range = 0.5
        self.petAI_surf_rh.inputs.sampling_units = "frac"


        #ELABORAZIONE VENOSA
        #conversione dicom->nifti
        self.venosa_conv = Node(Dcm2niix_mo(),name='venosa_conv')
        self.venosa_conv.inputs.out_filename ="venosa"
        #orientamento in convenzione radiologica
        venosa_reOrient=Node(Orient_mo(),name='venosa_reOrient')
        #separo le due fasi della phase contrast
        venosa_split=Node(Split(),name='venosa_split')
        venosa_split.inputs.dimension='t'
        #individuo la fase venosa dal modulo
        venosa_check=Node(VenosaCheck(), name='venosa_check')
        #segmento le strutture intracraniche nel modulo (migliore visualizzazione dell'osso)
        venosa_BET=Node(BET(),name='venosa_BET')
        venosa_BET.inputs.frac = 0.4
        venosa_BET.inputs.mask = True
        venosa_BET.inputs.threshold = True
        venosa_BET.inputs.surfaces = True
        #registrazione lineare modulo a ref
        venosa_modulo2ref_FLIRT=Node(FLIRT(), name='venosa_modulo2ref_FLIRT')
        venosa_modulo2ref_FLIRT.inputs.out_matrix_file  = "venosa2ref.mat"
        venosa_modulo2ref_FLIRT.inputs.cost = "mutualinfo"
        venosa_modulo2ref_FLIRT.inputs.searchr_x = [-90,90]
        venosa_modulo2ref_FLIRT.inputs.searchr_y = [-90,90]
        venosa_modulo2ref_FLIRT.inputs.searchr_z = [-90,90]
        venosa_modulo2ref_FLIRT.inputs.dof = 6
        venosa_modulo2ref_FLIRT.inputs.interp = "trilinear"
        #applico la maschera delle strutture intracraniche alla fase venosa
        venosa_inskull_mask=Node(ApplyMask(), name='venosa_inskull_mask')
        #trasformazione lineare fase venosa su ref
        self.venosa2ref_FLIRT=Node(FLIRT(), name='venosa2ref_FLIRT')
        self.venosa2ref_FLIRT.inputs.out_file  = "r-venosa_inskull.nii.gz"
        self.venosa2ref_FLIRT.inputs.interp = "trilinear"

        #ELABORAZIONE DTI
        #conversione dicom->nifti
        self.dti_conv = Node(Dcm2niix_mo(),name='dti_conv')
        self.dti_conv.inputs.out_filename ="dti"
        #estrazione immagine b0
        dti_nodif = Node(ExtractROI(),name='dti_nodif')
        dti_nodif.inputs.t_min=0
        dti_nodif.inputs.t_size=1
        dti_nodif.inputs.roi_file='nodif.nii.gz'
        #rimozione scalpo al b0
        nodif_BET = Node(BET(),name='nodif_BET')
        nodif_BET.inputs.frac = 0.3
        nodif_BET.inputs.robust = True
        nodif_BET.inputs.threshold = True
        nodif_BET.inputs.mask = True
        #correzione artefatti da movimento e eddy current
        dti_eddy = Node(EddyCorrect(),name='dti_eddy')
        dti_eddy.inputs.ref_num=0
        dti_eddy.inputs.out_file="data.nii.gz"
        #calcolo delle metriche base dti
        dti_dtifit = Node(DTIFit(),name='dti_dtifit')
        #calcolo del modello di trattografia
        dti_bedpostx = Node(BEDPOSTX5_mo(),name='dti_bedpostx')
        dti_bedpostx.inputs.n_fibres=2
        dti_bedpostx.inputs.rician=True
        #dti_bedpostx.inputs.num_threads=18
        dti_bedpostx.inputs.sample_every=25
        dti_bedpostx.inputs.n_jumps=1250
        dti_bedpostx.inputs.burn_in=1000
        #trasformazione lineare nodif nello spazio ref
        diff2ref_FLIRT = Node(FLIRT(), name='diff2ref_FLIRT')
        diff2ref_FLIRT.inputs.out_matrix_file  = "diff2ref.mat"
        diff2ref_FLIRT.inputs.cost = "corratio"
        diff2ref_FLIRT.inputs.searchr_x = [-90,90]
        diff2ref_FLIRT.inputs.searchr_y = [-90,90]
        diff2ref_FLIRT.inputs.searchr_z = [-90,90]
        diff2ref_FLIRT.inputs.dof = 6
        #calcolo di varie matrici di trasformazione derivate che serviranno per la trattografia
        ref2diff_convert = Node(ConvertXFM(),name='ref2diff_convert')
        ref2diff_convert.inputs.invert_xfm=True
        ref2diff_convert.inputs.out_file='ref2diff.mat'
        diff2mni_convert = Node(ConvertXFM(),name='diff2mni_convert')
        diff2mni_convert.inputs.concat_xfm=True
        diff2mni_convert.inputs.out_file='diff2mni.mat'
        mni2diff_convert = Node(ConvertXFM(),name='mni2diff_convert')
        mni2diff_convert.inputs.invert_xfm=True
        mni2diff_convert.inputs.out_file='mni2diff.mat'
        diff2mni_convertwarp = Node(ConvertWarp(),name='diff2mni_convertwarp')
        diff2mni_convertwarp.inputs.reference=mni
        diff2mni_convertwarp.inputs.out_file='diff2mni_warp.nii.gz'
        mni2diff_convertwarp = Node(ConvertWarp(),name='mni2diff_convertwarp')
        mni2diff_convertwarp.inputs.out_file='mni2diff_warp.nii.gz'


        #WORKFLOW CREATION
        self.t1 = Workflow_mo(name="t13d",base_dir="./")
        self.t1.connect(self.ref_conv, "converted_files", self.ref_reOrient, "in_file")
        self.t1.connect(self.ref_reOrient, "out_file", self.ref_BET, "in_file")

        self.mni = Workflow_mo(name="mni",base_dir="./")
        self.mni.add_nodes([ref2mni_FLIRT,ref2mni_FNIRT])
        self.connect(self.t1, "ref_BET.out_file", self.mni, "ref2mni_FLIRT.in_file")
        self.connect(self.t1, "ref_BET.out_file", self.mni, "ref2mni_FNIRT.in_file")
        self.mni.connect(ref2mni_FLIRT, "out_matrix_file", ref2mni_FNIRT, "affine_file")
        self.mni.connect(ref2mni_FNIRT, "fieldcoeff_file", ref2mni_INVWARP, "warp")
        self.connect(self.t1, "ref_BET.out_file", self.mni, "ref2mni_INVWARP.reference")

        # self.mni1 = Workflow_mo(name="mni1",base_dir="./")
        # self.mni1.add_nodes([ref2mni1_FLIRT,ref2mni1_FNIRT])
        # self.connect(self.t1, "ref_BET.out_file", self.mni1, "ref2mni1_FLIRT.in_file")
        # self.connect(self.t1, "ref_BET.out_file", self.mni1, "ref2mni1_FNIRT.in_file")
        # self.mni1.connect(ref2mni1_FLIRT, "out_matrix_file", ref2mni1_FNIRT, "affine_file")
        # self.mni1.connect(ref2mni1_FNIRT, "fieldcoeff_file", ref2mni1_INVWARP, "warp")
        # self.connect(self.t1, "ref_BET.out_file", self.mni1, "ref2mni1_INVWARP.reference")

        self.sym = Workflow_mo(name="sym",base_dir="./")
        self.sym.add_nodes([ref2sym_FLIRT,ref2sym_FNIRT])
        self.connect(self.t1, "ref_BET.out_file", self.sym, "ref2sym_FLIRT.in_file")
        self.connect(self.t1, "ref_BET.out_file", self.sym, "ref2sym_FNIRT.in_file")
        self.sym.connect(ref2sym_FLIRT, "out_matrix_file", ref2sym_FNIRT, "affine_file")
        self.sym.connect(ref2sym_FNIRT, "fieldcoeff_file", ref2sym_INVWARP, "warp")
        self.connect(self.t1, "ref_BET.out_file", self.sym, "ref2sym_INVWARP.reference")
        self.sym.connect(ref2sym_FNIRT, "warped_file", sym_SWAP, "in_file")
        self.sym.connect(sym_SWAP, "out_file", swap2sym_FLIRT, "in_file")
        self.sym.connect(ref2sym_FNIRT, "warped_file", swap2sym_FLIRT, "reference")
        self.sym.connect(sym_SWAP, "out_file", swap2sym_FNIRT, "in_file")
        self.sym.connect(swap2sym_FLIRT, "out_matrix_file", swap2sym_FNIRT, "affine_file")
        self.sym.connect(ref2sym_FNIRT, "warped_file", swap2sym_FNIRT, "ref_file")

        self.freesurfer = Workflow_mo(name="freesurfer",base_dir="./")
        self.freesurfer.add_nodes([self.reconAll])
        self.freesurfer.connect(self.reconAll,"rawavg",self.aparaseg2Volmgz,"template_file")
        self.freesurfer.connect([(self.reconAll, self.aparaseg2Volmgz, [(('aparc_aseg', getn, 0),'reg_header')])])
        self.freesurfer.connect([(self.reconAll, self.aparaseg2Volmgz, [(('aparc_aseg', getn, 0),'seg_file')])])
        self.freesurfer.connect(self.reconAll,"subjects_dir",self.aparaseg2Volmgz,"subjects_dir")
        self.freesurfer.connect(self.reconAll,"subject_id",self.aparaseg2Volmgz,"subject_id")
        self.freesurfer.connect(self.reconAll,"rawavg",aparaseg2Volnii,"template_file")
        self.freesurfer.connect([(self.reconAll, aparaseg2Volnii, [(('aparc_aseg', getn, 0),'reg_header')])])
        self.freesurfer.connect([(self.reconAll, aparaseg2Volnii, [(('aparc_aseg', getn, 0),'seg_file')])])
        self.freesurfer.connect(aparaseg2Volnii,"vol_label_file",lhwmROI,"in_file")
        self.freesurfer.connect(aparaseg2Volnii,"vol_label_file",rhwmROI,"in_file")
        self.freesurfer.connect(lhwmROI,"out_file",wmROI,"in_file")
        self.freesurfer.connect(rhwmROI,"out_file",wmROI,"operand_file")
        self.freesurfer.connect(aparaseg2Volnii,"vol_label_file",lhbgtROI,"in_file")
        self.freesurfer.connect(aparaseg2Volnii,"vol_label_file",rhbgtROI,"in_file")
        self.freesurfer.connect(lhbgtROI,"out_file",bgtROI,"in_file")
        self.freesurfer.connect(rhbgtROI,"out_file",bgtROI,"operand_file")

        self.flair = Workflow_mo(name="flair",base_dir="./")
        self.flair.connect(self.flair_conv, "converted_files", flair_reOrient, "in_file")
        self.flair.connect(flair_reOrient, "out_file", flair_BET, "in_file")
        self.flair.connect(flair_BET, "out_file", self.flair2ref_FLIRT, "in_file")

        # self.doMAP = Workflow_mo(name="doMAP",base_dir="./")
        # self.doMAP.add_nodes([doMAP_FAST,doMAP_flair2mni1])
        # self.connect(self.t1, "ref_BET.out_file",self.doMAP,"doMAP_FAST.in_files")
        # self.connect(self.flair,"flair_BET.out_file",self.doMAP,"doMAP_flair2mni1.in_file")
        # self.connect(self.mni1,"ref2mni1_FNIRT.fieldcoeff_file",self.doMAP,"doMAP_flair2mni1.field_file")
        # self.connect(self.flair,"flair2ref_FLIRT.out_matrix_file",self.doMAP,"doMAP_flair2mni1.premat")
        # self.doMAP.connect(doMAP_FAST,"restored_image",doMAP_restore2mni1,"in_file")
        # self.connect(self.mni1,"ref2mni1_FNIRT.fieldcoeff_file",self.doMAP,"doMAP_restore2mni1.field_file")
        # self.doMAP.connect([(doMAP_FAST, doMAP_gm2mni1, [(('partial_volume_files', getn, 1),'in_file')])])
        # self.connect(self.mni1,"ref2mni1_FNIRT.fieldcoeff_file",self.doMAP,"doMAP_gm2mni1.field_file")
        # self.doMAP.connect([(doMAP_FAST, doMAP_wm2mni1, [(('partial_volume_files', getn, 2),'in_file')])])
        # self.connect(self.mni1,"ref2mni1_FNIRT.fieldcoeff_file",self.doMAP,"doMAP_wm2mni1.field_file")
        # self.doMAP.connect(doMAP_flair2mni1,"out_file",doMAP_flairDIVref,"in_file")
        # self.doMAP.connect(doMAP_restore2mni1,"out_file",doMAP_flairDIVref,"operand_file")
        # self.doMAP.connect(doMAP_flairDIVref,"out_file",doMAP_cortexMask,"in_file")
        # self.doMAP.connect(doMAP_cortexMask,"out_file",doMAP_gmMask,"in_file")
        # self.doMAP.connect(doMAP_gm2mni1,"out_file",doMAP_gmMask,"mask_file")
        # self.doMAP.connect(doMAP_cortexMask,"out_file",doMAP_wmMask,"in_file")
        # self.doMAP.connect(doMAP_wm2mni1,"out_file",doMAP_wmMask,"mask_file")
        # self.doMAP.connect(doMAP_gmMask,"out_file",doMAP_gm_mean,"in_file")
        # self.doMAP.connect(doMAP_gmMask,"out_file",doMAP_gm_std,"in_file")
        # self.doMAP.connect(doMAP_wmMask,"out_file",doMAP_wm_mean,"in_file")
        # self.doMAP.connect(doMAP_wmMask,"out_file",doMAP_wm_std,"in_file")
        # self.doMAP.connect(doMAP_cortexMask,"out_file",doMAP_binaryFLAIR,"in_file")
        # self.doMAP.connect(doMAP_gm_mean,"out_stat",doMAP_binaryFLAIR,"seg_val_max")
        # self.doMAP.connect(doMAP_wm_mean,"out_stat",doMAP_binaryFLAIR,"seg_val_min")

        # self.doMAP.connect(doMAP_binaryFLAIR,"out_file",doMAP_convolution_flair,"in_file")

        # self.doMAP.connect(doMAP_convolution_flair,"out_file",doMAP_junction,"in_file")
        # self.doMAP.connect(doMAP_junction,"out_file",doMAP_junctionz,"in_file")




        self.mdc = Workflow_mo(name="mdc",base_dir="./")
        self.mdc.connect(self.mdc_conv, "converted_files", mdc_reOrient, "in_file")
        self.mdc.connect(mdc_reOrient, "out_file", mdc_BET, "in_file")
        self.mdc.connect(mdc_BET, "out_file", self.mdc2ref_FLIRT, "in_file")

        self.asl = Workflow_mo(name="asl",base_dir="./")
        self.asl.connect(self.asl_conv, "converted_files", asl_reOrient, "in_file")
        self.asl.connect(asl_reOrient, "out_file", asl_SMOOTH, "in_file")
        self.asl.connect(asl_reOrient, "out_file",asl2ref_FLIRT,"in_file")
        #self.asl.connect(self.asl_conv, "converted_files", asl_SMOOTH, "in_file")
        #self.asl.connect(self.asl_conv, "converted_files",asl2ref_FLIRT,"in_file")
        self.asl.connect(asl_SMOOTH, "out_file",aslsmooth2ref_FLIRT,"in_file")
        self.asl.connect(asl2ref_FLIRT, "out_matrix_file", aslsmooth2ref_FLIRT, "in_matrix_file")
        self.asl.connect(aslsmooth2ref_FLIRT, "out_file", self.asl_mask, "in_file")
        self.asl.connect(self.asl_mask, "out_file", asl2sym_APPLYWARP, "in_file")
        self.asl.connect(asl2sym_APPLYWARP, "out_file", aslsym_SWAP, "in_file")
        self.asl.connect(aslsym_SWAP, "out_file", asl_swapped_APPLYWARP, "in_file")
        self.asl.connect(aslsym_SWAP, "out_file", asl_AI, "in_file")
        self.asl.connect(asl_swapped_APPLYWARP, "out_file", asl_AI, "swapped_file")
        self.asl.connect(asl_AI, "out_file", self.asl_AI2ref, "in_file")

        self.pet = Workflow_mo(name="pet",base_dir="./")
        self.pet.connect(self.pet_ct_conv, "converted_files", pet_ct2ref_FLIRT, "in_file")
        self.pet.connect(self.pet_brain_conv, "converted_files", pet_brain2ct_FLIRT, "in_file")
        self.pet.connect(self.pet_ct_conv, "converted_files", pet_brain2ct_FLIRT, "reference")
        self.pet.connect(pet_ct2ref_FLIRT, "out_matrix_file", pet_convert_XFM, "in_file2")
        self.pet.connect(pet_brain2ct_FLIRT, "out_matrix_file", pet_convert_XFM, "in_file")
        self.pet.connect(self.pet_brain_conv, "converted_files", pet_brain2ref_FLIRT, "in_file")
        self.pet.connect(pet_convert_XFM, "out_file", pet_brain2ref_FLIRT, "in_matrix_file")
        self.pet.connect(pet_brain2ref_FLIRT, "out_file", self.pet_brain_SMOOTH, "in_file")
        self.pet.connect(self.pet_brain_SMOOTH, "out_file", pet_brain_sm_APPLYWARP, "in_file")
        self.pet.connect(pet_brain_sm_APPLYWARP, "out_file", petsym_SWAP, "in_file")
        self.pet.connect(petsym_SWAP, "out_file", pet_brain_swapped_APPLYWARP, "in_file")
        self.pet.connect(pet_brain_sm_APPLYWARP, "out_file", pet_AI, "in_file")
        self.pet.connect(pet_brain_swapped_APPLYWARP, "out_file", pet_AI, "swapped_file")
        self.pet.connect(pet_AI, "out_file", pet_AI2ref, "in_file")
        self.pet.connect(pet_AI2ref, "out_file", self.pet_AI_mask, "in_file")
        self.pet.connect(self.pet_brain_SMOOTH, "out_file", pet_zscore, "in_file")
        self.pet.connect(pet_zscore, "out_file", self.pet_zscore_mask, "in_file")

        self.venosa = Workflow_mo(name="venosa",base_dir="./")
        self.venosa.connect(self.venosa_conv, "converted_files", venosa_reOrient, "in_file")
        self.venosa.connect(venosa_reOrient, "out_file", venosa_split, "in_file")
        self.venosa.connect(venosa_split, "out_files", venosa_check, "in_files")
        self.venosa.connect(venosa_check, "out_file_modulo", venosa_BET, "in_file")
        self.venosa.connect(venosa_BET, "out_file", venosa_modulo2ref_FLIRT, "in_file")
        self.venosa.connect(venosa_check, "out_file_venosa", venosa_inskull_mask, "in_file")
        self.venosa.connect(venosa_BET, "inskull_mask_file", venosa_inskull_mask, "mask_file")
        self.venosa.connect(venosa_inskull_mask, "out_file", self.venosa2ref_FLIRT, "in_file")
        self.venosa.connect(venosa_modulo2ref_FLIRT, "out_matrix_file", self.venosa2ref_FLIRT, "in_matrix_file")

        self.dti_preproc = Workflow_mo(name="dti_preproc",base_dir="./")
        self.dti_preproc.connect(self.dti_conv, "converted_files", dti_nodif, "in_file")
        self.dti_preproc.connect(self.dti_conv, "converted_files", dti_eddy, "in_file")
        self.dti_preproc.connect(dti_nodif, "roi_file", nodif_BET, "in_file")
        self.dti_preproc.connect(dti_eddy, "eddy_corrected", dti_dtifit, "dwi")
        self.dti_preproc.connect(nodif_BET, "mask_file", dti_dtifit, "mask")
        self.dti_preproc.connect(self.dti_conv, "bvecs", dti_dtifit, "bvecs")
        self.dti_preproc.connect(self.dti_conv, "bvals", dti_dtifit, "bvals")
        self.dti_preproc.connect(nodif_BET, "out_file", diff2ref_FLIRT, "in_file")
        self.dti_preproc.connect(diff2ref_FLIRT, "out_matrix_file", ref2diff_convert, "in_file")
        self.dti_preproc.add_nodes([diff2mni_convert])
        self.dti_preproc.connect(diff2ref_FLIRT, "out_matrix_file", diff2mni_convert, "in_file2")
        self.dti_preproc.connect(diff2mni_convert, "out_file", mni2diff_convert, "in_file")
        self.dti_preproc.connect(diff2ref_FLIRT, "out_matrix_file", diff2mni_convertwarp, "premat")
        self.dti_preproc.connect(nodif_BET, "out_file", mni2diff_convertwarp, "reference")
        self.dti_preproc.connect(ref2diff_convert, "out_file", mni2diff_convertwarp, "postmat")
        self.dti_preproc.connect(dti_eddy, "eddy_corrected", dti_bedpostx, "dwi")
        self.dti_preproc.connect(nodif_BET, "mask_file", dti_bedpostx, "mask")
        self.dti_preproc.connect(self.dti_conv, "bvecs", dti_bedpostx, "bvecs")
        self.dti_preproc.connect(self.dti_conv, "bvals", dti_bedpostx, "bvals")
