from PySide6.QtWidgets import QMainWindow, QMessageBox, QFileDialog, QInputDialog, QLineEdit
from PySide6.QtGui import QAction, QIcon, QPixmap
from PySide6 import QtCore,QtWidgets
import os

from SWANi.UI.mainTAB import mainTAB

class mainWindow(QMainWindow):
    ptDirPath=""
    tabWidget=None
    
    def __init__(self, SWANglobalConfig):
        
        self.SWANglobalConfig=SWANglobalConfig
        
        super(mainWindow,self).__init__()
        
        self.inizializeUI()
        
        
        
        while self.SWANglobalConfig.getPatientsFolder()=="" or not os.path.exists(self.SWANglobalConfig.getPatientsFolder()):
            msgBox = QMessageBox()
            msgBox.setText("Choose the main working directory before start to use this application")
            msgBox.exec()
            self.setPatientsFolder()
            
        os.chdir(self.SWANglobalConfig.getPatientsFolder())

    @QtCore.Slot()
    def searchPtDir(self):
        fileDialog=QFileDialog()
        fileDialog.setDirectory(self.SWANglobalConfig.getPatientsFolder())
        folderPath = fileDialog.getExistingDirectory(self, 'Select a patient folder')
        if not os.path.exists(folderPath):
            return
        if not self.checkPtDir(folderPath):
            msgBox = QMessageBox()
            msgBox.setText("The selected folder does not contains patient data!")
            msgBox.exec()
            return
        os.chdir(folderPath)
        self.tabWidget.loadPt()
        self.statusBar().showMessage('Current patient: '+os.path.basename(folderPath))
        
    def getSuggestedPatientName(self):
        import re
        regex = re.compile('^'+self.SWANglobalConfig.getConfig("MAIN","patientsprefix")+'\d+$')
        fileList=[]
        for thisDir in os.listdir(self.SWANglobalConfig.getPatientsFolder()):
            if regex.match(thisDir): fileList.append(int(thisDir.replace(self.SWANglobalConfig['MAIN']['patientsprefix'],"")))
        
        if len(fileList)==0: return self.SWANglobalConfig['MAIN']['patientsprefix']+"1"
        return self.SWANglobalConfig['MAIN']['patientsprefix']+str(max(fileList)+1)
    

    def choseNewPtDir(self):
        text, ok = QInputDialog.getText(self, 'New patient', 
            'Write the name of the new patient:', QLineEdit.Normal, self.getSuggestedPatientName())
        
        if not ok: return
        
        ptName=str(text)
        
        if ptName=="": 
            msgBox = QMessageBox()
            msgBox.setText("Nome non valido: "+ptName)
            msgBox.exec()
            return
            
        if os.path.exists(os.path.join(self.SWANglobalConfig.getPatientsFolder(),ptName)):
            msgBox = QMessageBox()
            msgBox.setText("Esiste già un paziente con nome: "+ptName)
            msgBox.exec()
            return
            
        self.createNewPtDir(ptName)
        
    
    def setPatientsFolder(self):
        folderPath = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select the main working directory')
        if not os.path.exists(folderPath):
            return
        self.SWANglobalConfig.setConfig("MAIN","PatientsFolder",os.path.abspath(folderPath))
        self.SWANglobalConfig.save()
        os.chdir(folderPath)
        
    def createNewPtDir(self,ptName):    
        baseFolder=os.path.abspath(os.path.join(self.SWANglobalConfig.getPatientsFolder(),ptName))
        
        for folder in self.SWANglobalConfig["DEFAULTFOLDERS"]:
            os.makedirs(os.path.join(baseFolder,self.SWANglobalConfig['DEFAULTFOLDERS'][folder]),exist_ok=True)
        
        msgBox = QMessageBox()   
        msgBox.setText("New patient created in: "+baseFolder)
        msgBox.exec()
        os.chdir(baseFolder)
        self.tabWidget.loadPt()
        
    def checkPtDir(self,dirPath):
        for folder in self.SWANglobalConfig["DEFAULTFOLDERS"]:
            if not os.path.exists(os.path.join(dirPath,self.SWANglobalConfig["DEFAULTFOLDERS"][folder])):
               return False 
               
        return True
            
    
    def inizializeUI(self):
        self.resize(800,600)
        self.setWindowTitle('SWANi - Standardized Workflow for Advanced Neuro-imaging')    
        
        
        self.statusBar().showMessage('')
        
        button_action = QAction("&Load existing patient", self)
        button_action.setStatusTip("Load patient data from the main working directory")
        button_action.triggered.connect(self.searchPtDir)
        
        button_action2 = QAction("&Create new patient", self)
        button_action2.setStatusTip("Add a new patient in the main working directory")
        button_action2.triggered.connect(self.choseNewPtDir)
        
        menu = self.menuBar()
        file_menu = menu.addMenu("&File")
        file_menu.addAction(button_action)
        file_menu.addAction(button_action2)
        
        self.tabWidget=mainTAB(self.SWANglobalConfig,parent=self)
        self.setCentralWidget(self.tabWidget)
        
        self.setWindowIcon(QIcon(QPixmap(os.path.join(os.path.dirname(__file__),"icons/swan.png"))))
        
        self.show()
        
    def closeEvent(self, event):
        # evito la chiusura se il wf è in esecuzione
        if not hasattr(self,'tabWidget') or not hasattr(self.tabWidget,'SWAN_wf_thread') or not self.tabWidget.SWAN_wf_thread.isRunning():
            return super(mainWindow,self).closeEvent(event)
        else:
            self.tabWidget.start_SWAN_wf_thread()
            if not self.tabWidget.SWAN_wf_thread.isRunning():
                return super(mainWindow,self).closeEvent(event)
            
        event.ignore()