import configparser
import os
from SWANi.utils.APPLABELS import APPLABELS

class SWANiConfig(configparser.ConfigParser):
    configFile=os.path.abspath(os.path.join(os.path.expanduser("~"),"."+APPLABELS.APPNAME+"config"))
    INPUTLIST=['mr_t13d',
                            'mr_flair3d',
                            'mr_mdc',
                            'mr_venosa',
                            'mr_dti',
                            'mr_asl',
                            'ct_brain',
                            'mr_fmri',
                            'pet_brain']
    
    TRACTLIST=['af','ar','cst','fa','fx','ifo','ilf','or','uf']
                            
    def __init__(self):
        super(SWANiConfig,self).__init__()
        
        self.createDefaultConfig()
        
        if os.path.exists(self.configFile):
            self.read(self.configFile)
            
        self.save()
            
    def createDefaultConfig(self):
        
    
        self['MAIN'] = {'patientsfolder': '',
                        'patientsprefix': 'pt_',
                        'slicerPath':'',
                        'freesurfer':'true',
                        'hippoAmygLabels':'true'}
                        
        self['DEFAULTFOLDERS']={}
        for this in self.INPUTLIST:
            self['DEFAULTFOLDERS']['default_'+this+'_folder']='dicom/'+this+'/'
          
        self['DEFAULTNAMESERIES'] = {}
        for name in self.INPUTLIST:
            self['DEFAULTNAMESERIES']["Default_"+name+"_name"]=""
            
        self['DEFAULTTRACTS'] = {}
        for tract in self.TRACTLIST:
            if tract in ['cst','af','or']:
                self['DEFAULTTRACTS'][tract]= 'true'
            else:
                self['DEFAULTTRACTS'][tract]= 'false'

    
    def save(self):
        with open(self.configFile,"w") as openedFile:
            self.write(openedFile)
     
    def getConfig(self,section,name):
        try:
            return self[section][name]
        except NameError: 
            return ""
            
    def setConfig(self,section,name,value):
        self[section][name]=value
        
    def getPatientsFolder(self):
        return self.getConfig("MAIN","PatientsFolder")